﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Models;

namespace SushiBar
{
    public partial class FormStore : Form
    {
        private readonly List<StoreViewModel>? _listStores;
        private readonly IStoreLogic _logic;
        private readonly ILogger _logger;
        public int id { get; set; }
        public FormStore(ILogger<FormStore> logger, IStoreLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _listStores = logic.ReadList(null);
            _logic = logic;
            if (_listStores != null)
            {
                comboBoxName.DisplayMember = "StoreName";
                comboBoxName.ValueMember = "Id";
                comboBoxName.DataSource = _listStores;
                comboBoxName.SelectedItem = null;
            }
        }

        private void LoadData(bool extendDate = true)
        {
            try
            {
                var model = GetStore(extendDate ? id : Convert.ToInt32(comboBoxName.SelectedValue));
                if (model != null)
                {
                    comboBoxName.Text = model.StoreName;
                    textBoxAddress.Text = model.StoreAddress;
                    dateTimePicker.Text = Convert.ToString(model.OpeningDate);
                    maxSushi.Value = model.maxSushi;
                    dataGridView.Rows.Clear();
                    foreach (var el in model.Sushis.Values)
                    {
                        dataGridView.Rows.Add(el.Item1.SushiName, el.Item1.Price, el.Item2);
                    }
                }
                _logger.LogInformation("Load stores");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error on load");
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private IStoreModel? GetStore(int Id)
        {
            if (_listStores == null)
            {
                return null;
            }
            foreach (var elem in _listStores)
            {
                if (elem.Id == Id)
                {
                    return elem;
                }
            }
            return null;
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(comboBoxName.Text))
            {
                MessageBox.Show("Select store name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(textBoxAddress.Text))
            {
                MessageBox.Show("Fill in address", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Saving store");

            try
            {
                DateTime.TryParse(dateTimePicker.Text, out var dateTime);
                StoreBindingModel model = new()
                {
                    StoreName = comboBoxName.Text,
                    StoreAddress = textBoxAddress.Text,
                    OpeningDate = dateTime,
                    maxSushi = (int)maxSushi.Value
                };
                var searchModel = GetStore(id);
                bool operationResult;

                if (searchModel != null)
                {
                    model.Id = searchModel.Id;
                    operationResult = _logic.Update(model);
                }
                else
                {
                    operationResult = _logic.Create(model);
                }

                if (!operationResult)
                {
                    throw new Exception("Error on saving. Additional info below");
                }

                MessageBox.Show("Save is successful", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error on saving");
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void ComboBoxName_SelectedIndexChanged(object sender, EventArgs e)
        {
            LoadData(false);
        }

        private void FormStore_Load(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
